package	jp.co.sds_corp.util.db;

import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Timestamp;
import java.sql.Types;
import java.util.Properties;
import java.util.ResourceBundle;

import jp.co.sds_corp.util.config.IConfiguration;

/**
* ＤＢアクセス共通クラス。
<BR>
接続先の指定方法は、以下の3通りが可能です。
<P>
1.Propertiesファイルから取得<BR>
{@link #DbAccess(ResourceBundle)},{@link #DbAccess(String)}を参照してください。
<PRE>
propertiesファイルは以下の形式です。
JNDIから取得する場合
DB_JNDI_URL=ap1.sds-corp.co.jp
DB_SOURCE=jdbc/vfx2DataSource
DB_USER=user
DB_PASSWORD=password
DB_ENCRYPTED=true
DB_ENCODE=EUC-JP

JDBCを直接使用する場合
DB_URL=jdbc:oracle:oci8:@DB
DB_DRIVER=oracle.jdbc.OracleDriver
DB_USER=user
DB_PASSWORD=password
DB_ENCRYPTED=true
DB_ENCODE=EUC-JP
DB_JAR=../lib/ojdbc14.jar-JP

また、キーをつけての指定も可能です。
dbkey.DB_CONTEXT=
dbkey.DB_JNDI_URL=
dbkey.DB_SOURCE=
dbkey.DB_URL=jdbc:oracle:oci8:@DB
dbkey.DB_DRIVER=oracle.jdbc.OracleDriver
dbkey.DB_USER=user
dbkey.DB_PASSWORD=password
dbkey.DB_ENCRYPTED=true
dbkey.DB_ENCODE=EUC-JP
dbkey.DB_JAR=../lib/ojdbc14.jar-JP
</PRE>
<P>
2.XMLから取得<BR>
{@link #DbAccess(IConfiguration)}を参照してください。
<PRE>
XMLは以下の形式です。
dbキーを指定する場合
&lt;db type="properties"&gt;
    &lt;key&gt;db_key&lt;/key&gt;
&lt;/db&gt;

JNDIを指定使用する場合
&lt;db type="jndi"&gt;
    &lt;context&gt;org.jnp.interfaces.NamingContextFactory&lt;/context&gt;
    &lt;url&gt;localhost&lt;/url&gt;
    &lt;datasource&gt;java:/DefaultDS&lt;/datasource&gt;
    &lt;user&gt;user&lt;/user&gt;
    &lt;password&gt;password&lt;/password&gt;
    &lt;encrypted&gt;true&lt;/encrypted&gt;
    &lt;encode&gt;MS932&lt;/encode&gt;
&lt;/db&gt;

JDBCドライバを直接指定する場合
&lt;db type="jdbc"&gt;
    &lt;driver&gt;com.sybase.jdbc2.jdbc.SybDriver&lt;/driver&gt;
    &lt;url&gt;jdbc:sybase:Tds:localhost:4100/db&lt;/url&gt;
    &lt;user&gt;user&lt;/user&gt;
    &lt;password&gt;password&lt;/password&gt;
    &lt;encrypted&gt;true&lt;/encrypted&gt;
    &lt;encode&gt;MS932&lt;/encode&gt;
	&lt;jar&gt;./lib/ojdbc14.jar&lt;/jar&gt;
    &lt;properties key="some" value="props"/&gt;
&lt;/db&gt;
</PRE>
<P>
3.自分で{@link DbInfo}クラスを構築<BR>
{@link #DbAccess(DbInfo)}を参照してください。
* @author 小見山　聡
*/
public class DbAccess
{
	/** ＳＱＬ */
	String	sqlstr = null;

	/** コネクション */
	Connection conn = null;
	/** 結果セット */
	ResultSet rset = null;
	/** ステートメント */
	PreparedStatement stmt = null;
	/** 文字列のtrim */
	boolean trim = true;
	/** 自動Commitを使用するか */
	boolean	useAutoCommit = true;
	/** SQL文出力フラグ */
	boolean	sqlOutput = true;

	/** ＤＢ接続情報 */
	DbInfo	inf = null;

	private static final String SQL99_DUPLICATE = "23000";

	/**
	* コンストラクタ
	* デフォルトのＤＢの接続情報を取得する
	*/
	public DbAccess() {
		inf = new DbInfo();
	}

	/**
	* 指定されたpropertiesファイルで初期化します。
	* @param	prop	properiesファイル
	*/
	public DbAccess( ResourceBundle prop ) {
		inf = new DbInfo( prop );
	}

	/**
	* コンストラクタ
	* 指定されたキーによりＤＢの接続情報を取得する
	* @param	key		このキーでprpertiesファイルを検索する
	*/
	public DbAccess( String key ) {
		inf = new DbInfo( key );
	}

	/**
	* 指定された情報で初期化します。
	* @param	inf		DB接続情報
	*/
	public DbAccess( DbInfo inf ) {
		this.inf = inf;
	}

	/**
	* 指定されたXMLで初期化します。
	* @param	config	DB接続情報
	*/
	public DbAccess( IConfiguration config ) throws Exception {
		inf = new DbInfo( config );
	}

	public DbInfo getDbInfo() {
		return inf;
	}

	/**
	* ＤＢの接続情報により、ＤＢにコネクトする
	*/
	public void connect() throws DbException {
		rset = null;
		stmt = null;
		DbConnectionFactory.setSqlOutput( isSqlOutput() );
		conn = DbConnectionFactory.getConnection( inf );
		setAutoCommit( false );
	}
	/**
	* ＤＢの接続情報により、ＤＢにコネクトする
	*/
	public void connect( Properties prop ) throws DbException {
		inf.setProperties( prop );
		connect();
	}

	/**
	* ドライバを指定してDBサーバと接続します。
	* @param	drv		ドライバ
	*/
	public void connect( Driver drv ) throws DbException {
		rset = null;
		stmt = null;
		DbConnectionFactory.setSqlOutput( isSqlOutput() );
		conn = DbConnectionFactory.getConnection( drv, inf );
	}
	/**
	* ドライバを指定してDBサーバと接続します。
	* @param	drv		ドライバ
	* @param	prop	接続プロパティ
	*/
	public void connect( Driver drv, Properties prop ) throws DbException {
		inf.setProperties( prop );
		connect( drv );
	}

	/**
	* 自動COMMITの設定
	* @param	bFlg	自動COMMITのon/off
	*/
	public void setAutoCommit( boolean bFlg ) throws DbException {
		if ( !useAutoCommit ) {
			return;
		}
		try {
			conn.setAutoCommit( bFlg );
		} catch ( SQLException ex ) {
			throw new DbException( "setAutoCommit() error", "", ex );
		}
	}

	/**
	* 自動Commitを使用するかを設定します。
	* @param	var		設定するデータ
	*/
	public void setUseAutoCommit( boolean var ) {
		this.useAutoCommit = var;
	}
	/**
	* 自動Commitを使用するかを取得します。
	* @return	自動Commitを使用するか
	*/
	public boolean getUseAutoCommit() {
		return useAutoCommit;
	}

	/**
	* 文字列をtrimするかを設定します。
	* @param	var		設定するデータ
	*/
	public void setTrim( boolean var ) {
		this.trim = var;
	}
	/**
	* 文字列をtrimするかを取得します。
	* @return	文字列をtrimするか
	*/
	public boolean isTrim() {
		return trim;
	}

	/**
	* 問い合わせの実行
	* @param	sql		問い合わせ内容
	*/
	public void executeQuery( String sql ) throws DbException {
		rset = null ;
		try {
			sqlstr = sql;
			//ResultSetの作成
			if ( stmt != null ) {
				stmt.close();
				stmt = null;
			}
			stmt = getStatement( sql );
			rset = stmt.executeQuery();
		} catch ( SQLException ex ) {
			throw new DbException( "executeQuery() error", sqlstr, ex );
		}
	}

	/**
	* 問い合わせの準備
	* @param	sql		問い合わせ内容
	*/
	public void prepareQuery( String sql ) throws DbException {
		rset = null ;
		try {
			sqlstr = sql;
			//ResultSetの作成
			if ( stmt != null ) {
				stmt.close();
				stmt = null;
			}
			stmt = getStatement( sql );
		} catch ( Exception ex ) {
			if ( stmt != null ) {
				((DbPreparedStatement )stmt).printParameter();
			}
			throw new DbException( "prepareQuery() error", sqlstr, ex );
		}
	}

	/**
	* 準備された問い合わせの実行
	*/
	public void executePreparedQuery() throws DbException {
		rset = null ;
		try {
			rset = stmt.executeQuery();
		} catch ( SQLException ex ) {
			if ( stmt != null ) {
				((DbPreparedStatement )stmt).printParameter();
			}
			throw new DbException( "executePreparedQuery() error", sqlstr, ex );
		}
	}

	/**
	* 検索結果の取得。
	* 検索結果は、DbColumn変数に設定される
	* @param	tbl		検索結果の格納バッファ
	* @return	成功：true 失敗:false
	*/
	public boolean fetch( DbTable tbl ) throws DbException {
		if ( rset == null ) {
			return false;
		}
		try {
			if ( !tbl.isInitColumn() ) {
				tbl.setColumnInfo( rset );
			}
			if ( rset.next() != true ) {
				rset = null;
				return false;
			}
		} catch ( Exception ex ) {
			rollback();
			String msg = "fetch() error";
			throw new DbException( msg, sqlstr, ex );
		}
		return fetch( rset, tbl );
	}

	/**
	* 検索結果の取得。
	* 検索結果は、DbColumn変数に設定される
	* @param	tbl		検索結果の格納バッファ
	* @return	成功：true 失敗:false
	*/
	public boolean fetch( ResultSet rset, DbTable tbl ) throws DbException {
		if ( rset == null ) {
			return false;
		}
		String colName = null;
		int colType = -1;
		try {
			if ( !tbl.isInitColumn() ) {
				tbl.setColumnInfo( rset );
			}
			int nCol = tbl.getColumnCount();
			for ( int nLoop = 0; nLoop < nCol; nLoop++ ) {
				colName = null;
				colType = -1;
				DbColumn col = tbl.getColumn( nLoop );
				colName = col.getName();
				colType = col.getType();
				switch ( col.getType()) {
				case Types.VARCHAR:
				case Types.CHAR:
				case Types.NCHAR:
				case Types.NVARCHAR:
				case Types.LONGVARCHAR:
				{
					String res = rset.getString( nLoop+1 );
					if ( res != null ) {
						if ( inf.getEncode() != null ) {
							if ( inf.getEncode().equals( "SJIS" )) {
								res = res.replace( '\uff0d', '\u2212' );
							} else
							if ( inf.getEncode().equals( "MS932" ) ||
								inf.getEncode().equals( "Windows-31J" )) {
								res = res.replace( '\u301c', '\uff5e' );
							}
						}
					}
					if ( res == null || rset.wasNull()) {
						col.setObject( null );
					} else {
						if ( isTrim() ) {
							col.setObject( res.trim() );
						} else {
							col.setObject( res );
						}
					}
/*
					if ( isTrim()) {
						col.setObject( rset.getString( nLoop+1 ).trim());
					} else {
						col.setObject( rset.getString( nLoop+1 ) );
					}

					String	str = new String( rset.getBytes( nLoop+1 ), "eucjis" );
					InputStream is = rset.getBinaryStream( nLoop+1 );
					byte[]	buf = new byte[4096];
					is.read( buf );
					String str = new String( buf, "eucjis" );
					if ( isTrim()) {
						col.setObject( str.trim());
					} else {
						col.setObject( str );
					}
*/
					break;
				}
				case Types.BIT:
				case Types.BOOLEAN:
				{
					boolean res = rset.getBoolean( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( Boolean.valueOf( res ));
					}
					break;
				}
				case Types.TINYINT:
				case Types.BIGINT:
				case Types.INTEGER:
				case Types.SMALLINT:
				{
					int res = rset.getInt( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( Integer.valueOf( res ));
					}
					break;
				}
				case Types.NUMERIC:
				case Types.DECIMAL:
				{
					BigDecimal res = rset.getBigDecimal( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( res );
					}
					break;
				}
				case Types.REAL:
				case Types.FLOAT:
				{
					float res = rset.getFloat( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( new Float( res ));
					}
					break;
				}
				case Types.DOUBLE:
				{
					double res = rset.getDouble( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( new Double( res ));
					}
					break;
				}
				case Types.DATE:
				{
					java.sql.Timestamp res = rset.getTimestamp( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( new Timestamp( res.getTime() ));
					}
					break;
				}
				case Types.TIME:
				{
					java.sql.Time res = rset.getTime( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( new Timestamp( res.getTime() ));
					}
					break;
				}
				case Types.TIMESTAMP:
				{
					java.sql.Timestamp res = rset.getTimestamp( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( new Timestamp( res.getTime() ));
					}
					break;
				}
				case Types.BINARY:
				case Types.VARBINARY:
				case Types.LONGVARBINARY:
				case Types.OTHER:
				case Types.JAVA_OBJECT:
				case Types.DISTINCT:
				case Types.STRUCT:
				case Types.ARRAY:
				case Types.REF:
				case Types.DATALINK:
				{
					Object res = rset.getObject( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( res );
					}
					break;
				}
				case Types.BLOB:
				{
					Object res = rset.getBlob( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( res );
					}
					break;
				}
				case Types.CLOB:
				{
					Object res = rset.getClob( nLoop+1 );
					if ( rset.wasNull()) {
						col.setObject( null );
					} else {
						col.setObject( res );
					}
					break;
				}
				case Types.NULL:
					col.setObject( null );
					break;
				default:
					throw( new DbException( "type error:" + col.getType() ));
				}
			}
		} catch ( Exception ex ) {
			rollback();
			String msg = "fetch() error";
			if ( colName != null ) {
				msg += " column=[" + colName + ":" + colType + "]";
			}
			throw new DbException( msg, sqlstr, ex );
		}
		return true;
	}

	/**
	* 準備された問い合わせの実行
	* @return	最初の結果が ResultSet オブジェクトの場合は true。
	*			更新カウントであるか、または結果がない場合は false
	*/
	public boolean executePrepared() throws DbException {
		boolean bRet = true ;
		try {
			bRet = stmt.execute();
			if ( bRet ) {
				rset = stmt.getResultSet();
			}
		} catch ( SQLException ex ) {
			if ( stmt != null ) {
				((DbPreparedStatement )stmt).printParameter();
			}
			if ( ex.getSQLState() != null &&
				ex.getSQLState().equals( SQL99_DUPLICATE ))
			{
				throw new DuplicateException( "executePrepared() error "
							+ "SQLState=" + ex.getSQLState(), sqlstr, ex );
			}
			throw new DbException( "executePrepared() error", sqlstr, ex );
		}
		return bRet;
	}

	/**
	* 更新、挿入の実行
	* @param	sql		引数 update,insert,deleteの文字列
	* @return	最初の結果が ResultSet オブジェクトの場合は true。
	*			更新カウントであるか、または結果がない場合は false
	*/
	public boolean execute( String sql ) throws DbException {
		boolean bRet = true ;
		try {
			sqlstr = sql;
			//Statement の作成
			if ( stmt != null ) {
				stmt.close();
				stmt = null;
			}
			stmt = getStatement( sql );
			bRet = stmt.execute();
			if ( bRet ) {
				rset = stmt.getResultSet();
			}
		} catch ( SQLException ex ) {
			if ( ex.getSQLState() != null &&
				ex.getSQLState().equals( SQL99_DUPLICATE ))
			{
				throw new DuplicateException( "execute() error "
							+ "SQLState=" + ex.getSQLState(), sqlstr, ex );
			}
			throw new DbException( "execute() error", sqlstr, ex );
		}
		return bRet;
	}

	/**
	* 次の結果の取得
	* @return	最初の結果が ResultSet オブジェクトの場合は true。
	*			更新カウントであるか、または結果がない場合は false
	*/
	public boolean getMoreResults() throws DbException {
		boolean bRet = false;
		try {
			bRet = stmt.getMoreResults();
			if ( bRet ) {
				rset = stmt.getResultSet();
			}
		} catch ( SQLException ex ) {
			throw new DbException( "getMoreResults() error", sqlstr, ex );
		}
		return bRet;
	}

	/**
	* SQL Warningを取得します。
	* @return	SQL警告
	*/
	public SQLWarning getWarnings() throws DbException {
		SQLWarning sw = null;
		try {
			if ( stmt != null ) {
				sw = stmt.getWarnings();
			}
		} catch ( SQLException ex ) {
			throw new DbException( "getWarning() error", sqlstr, ex );
		}
		return sw;
	}

	/**
	* SQL Warningをクリアします。
	*/
	public void clearWarnings() throws DbException {
		try {
			if ( stmt != null ) {
				stmt.clearWarnings();
			}
		} catch ( SQLException ex ) {
			throw new DbException( "clearWarning() error", sqlstr, ex );
		}
	}

	/**
	* 更新、挿入の実行
	* @return		実行対象の行の数
	*/
	public int getUpdateCount() throws DbException {
		int rtn = -1;
		try {
			rtn = stmt.getUpdateCount();
		} catch ( SQLException ex ) {
			throw new DbException( "getUpdateCount() error", sqlstr, ex );
		}
		return rtn;
	}

	ParameterMetaData md = null;
	/**
	* パラメータを取得します。
	* @return	true:パラメータ有り false:パラメータ無し
	*/
	public boolean getParameter() throws DbException {
		boolean bRet = false;
		try {
			md = stmt.getParameterMetaData();
			if ( md != null ) {
				bRet = true;
			}
		} catch ( SQLException ex ) {
			throw new DbException( "getParameterMetaData() error", sqlstr, ex );
		}
		return bRet;
	}

	/**
	* SQLの取り消し
	*/
	public void cancel() throws DbException {
		try {
			if ( rset != null ) {
				rset.close();
				rset = null;
			}
			if ( stmt != null ) {
//				stmt.cancel();
				stmt.close();
				stmt = null;
			}
		} catch ( SQLException ex ) {
			throw new DbException( "cancel() error", sqlstr, ex );
		}
	}

	/**
	* データベーストランザクションのコミット
	*/
	public void commit() throws DbException {
		try {
			cancel();
			conn.commit() ;
		} catch ( SQLException ex ) {
			throw new DbException( "commit() error", sqlstr, ex );
		}
	}

	/**
	* データベーストランザクションのロールバック
	*/
	public void rollback() throws DbException {
		try {
			cancel();
			conn.rollback() ;
		} catch ( SQLException ex ) {
			throw new DbException( "rollback() error", sqlstr, ex );
		}
	}

	/**
	* データベース接続の終了
	*/
	public void disconnect() throws DbException {
		try {
			cancel();
			if ( conn != null && !conn.isClosed()) {
				conn.close() ;
			}
		} catch ( SQLException ex ) {
			throw new DbException( "disconnect() error", sqlstr, ex );
		}
	}

	/**
	* Connectionの取得
	* @return		Connection
	*/
	public Connection getConnection() {
		return conn;
	}

	/**
	* Statementの取得
	* @return		PreparedStatement
	*/
	public PreparedStatement getStatement() throws DbException {
		return stmt;
	}

	/**
	* SQL を実行して結果のStatementを取得します。
	* @param	sql		SQL文
	* @return	実行結果
	*/
	public PreparedStatement getStatement( String sql  )
		throws DbException
	{
		PreparedStatement st;
		try {
			st = conn.prepareStatement( sql );
//			st = conn.prepareCall( sql );
		} catch ( SQLException ex ) {
			throw new DbException( "getStatement(prepare) error", sqlstr, ex );
		}
		return st;
	}

	/**
	* 結果セットの取得
	*/
	public ResultSet getResultSet() {
		return rset;
	}
	/**
	* 結果セットの設定
	*/
	public void setResultSet( ResultSet var ) {
		rset = var;
	}

	/**
	* row数の取得
	* @param where  countする条件。nullまたは""ならテーブルの全レコードが対象
	* @return       0以上:行数 0未満:エラー
	*/
	public int countRow( String tbl, String where ) throws DbException {
		if ( tbl == null ) {
			throw new DbException( "table name not set" );
		}

		String sql = "select count(*) from " + tbl;
		if (( where != null ) && ( where.length() > 0 )) {
			sql = sql + " " + where;
		}

		int nRow = -1;
		try {
			executeQuery( sql );

			boolean bRet = rset.next();
			if ( bRet != true ) {
				return -1;
			}

			nRow = rset.getInt( 1 );
			cancel();
		} catch ( SQLException ex ) {
			throw new DbException( "countRow() error", sqlstr, ex );
		}
		return nRow;
	}

	/**
	* テーブルから全カラムを取得するSQL文を作成し、実行します
	* @param	tbl		テーブル情報
	* @param	where	where/order by/group by句
	*/
	public void selectAll( DbTable tbl, String where ) throws DbException {
		String  sql = tbl.getSelectSql( where );
		executeQuery( sql );
	}

	/**
	* テーブルから、キーで指定されたレコードの
	* 全カラムを取得するSQL文を実行します
	* @return	true:正常 false:該当データなし
	*/
	public boolean selectAllByKey( DbTable tbl ) throws DbException {
		String sql = tbl.getSelectByKeySql();
		executeQuery( sql );
		if ( fetch( tbl ) != true ) {
			cancel();
			return false;
		}
		cancel();
		return true;
	}

	/**
	* テーブルにオブジェクトの内容をinsertします
	* @param	tbl		テーブル情報
	* @return	true:正常 false:該当データなし
	*/
	public boolean insert( DbTable tbl ) throws DbException {
		prepareInsert( tbl );
		boolean bRet = insertPrepared( tbl );
		return bRet;
	}

	/**
	* テーブルにオブジェクトの内容をinsertします
	* @param	tbl		テーブル情報
	*/
	public void prepareInsert( DbTable tbl ) throws DbException {
		String  sql = tbl.getInsertSql();
		prepareQuery( sql );
	}

	/**
	* テーブルにオブジェクトの内容をinsertします
	* @param	tbl		テーブル情報
	* @return	true:正常 false:該当データなし
	*/
	public boolean insertPrepared( DbTable tbl ) throws DbException {
		try {
			int nPos = 1;
			for ( int i = 1; i <= tbl.getColumnCount(); i++ ) {
				if ( !tbl.isAuditColumn( tbl.getColumn( i-1 ).getName() )) {
					try {
						getStatement().setObject( nPos++, tbl.getObject( i-1 ));
					} catch ( SQLException ex ) {
						throw new DbException( "insert() error,"
							+ "col=" + tbl.getColumn( i-1 ).getName(),
							sqlstr, ex );

					}
				}
			}
			boolean bRet = executePrepared();
			return bRet;
		} catch ( DbException ex ) {
			if ( stmt != null ) {
				((DbPreparedStatement )stmt).printParameter();
			}
			throw( ex );
		}
	}

	/**
	* テーブルから、キーで指定されたレコードを削除します
	* @param	tbl		テーブル情報
	* @return	true:正常 false:該当データなし
	*/
	public boolean delete( DbTable tbl ) throws DbException {
		String sql = tbl.getDeleteSql();
		boolean bRet = execute( sql );
		return bRet;
	}

	/**
	* SQL文出力フラグを設定します。
	* @param	var		SQL文出力フラグ
	*/
	public void setSqlOutput( boolean var ) {
		this.sqlOutput = var;
	}
	/**
	* SQL文出力フラグを取得します。
	* @return	SQL文出力フラグ
	*/
	public boolean isSqlOutput() {
		return this.sqlOutput;
	}

	/**
	* この出力先の文字列表現を返します。
	* @return   この出力先の文字列表現
	*/
	public String toString() {
		return inf.toString();
	}

public static void main( String[] args ) {
	try {
		ResourceBundle prop = ResourceBundle.getBundle( "db" );
		DbAccess db = new DbAccess( prop );
		db.connect();
		db.disconnect();
	} catch ( Exception ex ) {
		ex.printStackTrace();
	}
}
}
