package	jp.co.sds_corp.util.db;

import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.NClob;
import java.sql.PreparedStatement;
import java.sql.SQLClientInfoException;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.SQLXML;
import java.sql.Savepoint;
import java.sql.Statement;
import java.sql.Struct;
import java.util.Date;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Executor;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* Connectionのラッパクラスです。
* SQL実行時に、SQL文をログファイルに出力します。
* また、SQL実行時に生成されるStatement,PreparedStatement,CallableStatementを
* それぞれラップして返します。
* @author   小見山　聡
*/
public class DbConnection implements Connection
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	Connection con;
	/** SQL文出力フラグ */
	boolean	sqlOutput = true;

	protected DbConnection( Connection con ) {
		this.con = con;
	}

	/**
	* SQL文出力フラグを設定します。
	* @param	var		SQL文出力フラグ
	*/
	public void setSqlOutput( boolean var ) {
		this.sqlOutput = var;
	}
	/**
	* SQL文出力フラグを取得します。
	* @return	SQL文出力フラグ
	*/
	public boolean isSqlOutput() {
		return this.sqlOutput;
	}

    public Statement createStatement() throws SQLException {
		Statement stmt = con.createStatement();
		DbStatement ret = new DbStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		return ret;
	}
    public PreparedStatement prepareStatement(String sql) throws SQLException {
		PreparedStatement stmt = con.prepareStatement( sql );
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}
    public CallableStatement prepareCall(String sql) throws SQLException {
		CallableStatement stmt = con.prepareCall(sql);
		return stmt;
	}
    public String nativeSQL(String sql) throws SQLException {
		return con.nativeSQL(sql);
	}
    public void setAutoCommit(boolean autoCommit) throws SQLException {
		con.setAutoCommit(autoCommit);
	}
    public boolean getAutoCommit() throws SQLException {
		return con.getAutoCommit();
	}
    public void commit() throws SQLException {
		Date start = new Date();
		con.commit();
		Date now = new Date();
		long time = now.getTime() - start.getTime();
		logger.sql( "DB commit elapsed time " + time + " ms.");
	}
    public void rollback() throws SQLException {
		Date start = new Date();
		con.rollback();
		Date now = new Date();
		long time = now.getTime() - start.getTime();
		logger.sql( "DB rollback elapsed time " + time + " ms.");
	}
    public void close() throws SQLException {
		con.close();
	}
    public boolean isClosed() throws SQLException {
		return con.isClosed();
	}

    //======================================================================
    // Advanced features:

    public DatabaseMetaData getMetaData() throws SQLException {
		return con.getMetaData();
	}
    public void setReadOnly(boolean readOnly) throws SQLException {
		con.setReadOnly(readOnly);
	}
    public boolean isReadOnly() throws SQLException {
		return con.isReadOnly();
	}
    public void setCatalog(String catalog) throws SQLException {
		con.setCatalog(catalog);
	}
    public String getCatalog() throws SQLException {
		return con.getCatalog();
	}
    public void setTransactionIsolation(int level) throws SQLException {
		con.setTransactionIsolation(level);
	}
    public int getTransactionIsolation() throws SQLException {
		return con.getTransactionIsolation();
	}
    public SQLWarning getWarnings() throws SQLException {
		return con.getWarnings();
	}
    public void clearWarnings() throws SQLException {
		con.clearWarnings();
	}

    //--------------------------JDBC 2.0-----------------------------
    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
		Statement stmt = con.createStatement(resultSetType, resultSetConcurrency);
		DbStatement ret = new DbStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		return ret;
	}
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
		PreparedStatement stmt = con.prepareStatement(sql, resultSetType, resultSetConcurrency);
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
		CallableStatement stmt = con.prepareCall(sql, resultSetType, resultSetConcurrency);
		return stmt;
	}
    public Map<String,Class<?>> getTypeMap() throws SQLException {
		return con.getTypeMap();
	}
    public void setTypeMap(Map<String,Class<?>> map) throws SQLException {
		con.setTypeMap(map);
	}

    //--------------------------JDBC 3.0-----------------------------

    public void setHoldability(int holdability) throws SQLException {
		con.setHoldability(holdability);
	}
    public int getHoldability() throws SQLException {
		return con.getHoldability();
	}
    public Savepoint setSavepoint() throws SQLException {
		return con.setSavepoint();
	}
    public Savepoint setSavepoint(String name) throws SQLException {
		return con.setSavepoint(name);
	}
    public void rollback(Savepoint savepoint) throws SQLException {
		con.rollback(savepoint);
	}
    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
		con.releaseSavepoint(savepoint);
	}
    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		Statement stmt = con.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
		DbStatement ret = new DbStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		return ret;
	}
    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		PreparedStatement stmt = con.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}
    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
		CallableStatement stmt = con.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		return stmt;
	}
    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
		PreparedStatement stmt = con.prepareStatement(sql, autoGeneratedKeys);
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}
    public PreparedStatement prepareStatement(String sql, int columnIndexes[]) throws SQLException {
		PreparedStatement stmt = con.prepareStatement(sql, columnIndexes);
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}
    public PreparedStatement prepareStatement(String sql, String columnNames[]) throws SQLException {
		PreparedStatement stmt = con.prepareStatement(sql, columnNames);
		DbPreparedStatement ret = new DbPreparedStatement( stmt );
		ret.setSqlOutput( isSqlOutput() );
		ret.setSql( sql );
		return ret;
	}

    public Clob createClob() throws SQLException {
		return con.createClob();
	}
    public Blob createBlob() throws SQLException {
		return con.createBlob();
	}
    public NClob createNClob() throws SQLException {
		return con.createNClob();
	}
    public SQLXML createSQLXML() throws SQLException {
		return con.createSQLXML();
	}
	public boolean isValid(int timeout) throws SQLException {
		return con.isValid(timeout);
	}
	public void setClientInfo(String name, String value) throws SQLClientInfoException {
		con.setClientInfo(name, value);
	}
	public void setClientInfo(Properties properties) throws SQLClientInfoException {
		con.setClientInfo(properties);
	}
	public String getClientInfo(String name) throws SQLException {
		return con.getClientInfo(name);
	}
	public Properties getClientInfo() throws SQLException {
		return con.getClientInfo() ;
	}
	public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
		return con.createArrayOf(typeName, elements);
	}
	public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
		return con.createStruct(typeName, attributes);
	}

    //-------------------------- Java7 -----------------------------
	@Override
	public int getNetworkTimeout() throws SQLException {
		return con.getNetworkTimeout();
	}

	@Override
	public void setNetworkTimeout(Executor executor, int milliseconds)
		throws SQLException
	{
		con.setNetworkTimeout(executor, milliseconds);
	}

	@Override
	public void abort(Executor executor) throws SQLException {
		con.abort(executor);
	}

	@Override
	public String getSchema() throws SQLException {
		return con.getSchema();
	}

	@Override
	public void setSchema(String schema) throws SQLException {
		con.setSchema(schema);
	}

    //======================================================================
    // Wrapper:
	public <T> T unwrap(Class<T> iface) throws SQLException {
		return con.unwrap(iface);
	}
    public boolean isWrapperFor(Class<?> iface) throws SQLException {
		return con.isWrapperFor(iface);
	}

}
