package	jp.co.sds_corp.util.db;

import java.util.Iterator;
import java.util.MissingResourceException;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;

import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.config.UnspecifiedPropertyException;

/**
* ＤＢ接続情報クラス
* @author 小見山　聡
*/
public class DbInfo
{
	/** 接続タイプ */
	public enum Type {
		/** 接続タイプ:DatSource */
		DATASOURCE,
		/** 接続タイプ:JDBC */
		JDBC
	};

	/** 接続タイプ */
	Type	type;
	/** Context Factory */
	String	context;
	/** JNDI URL */
	String	jndi;
	/** DataSource */
	String	source;
	/** ドライバ */
	String	driver;
	/** ＤＢ接続先 */
	String	url;
	/** ユーザ */
	String	user;
	/** パスワード */
	String	password;
	/** jar */
	String	jar;
	/** パスワー暗号化 */
	boolean	encrypted = false;
	private static final String DEFAULT_KEY = "jp.co.sds_corp.util.db";
	/** パスワー暗号化キー */
	String	encryptKey = DEFAULT_KEY;
	/** 文字エンコード */
	String	encode;
	/** 接続プロパティ */
	Properties	props = new Properties();
	ResourceBundle infoProp = null;

	/**
	* コンストラクタ
	* 標準の接続先への情報を取得する
	*/
	public DbInfo() {
		readProperties( null, null );
	}

	/**
	* コンストラクタ
	* 指定されたキーで、接続先への情報を取得する
	* @param	sub			キー
	*/
	public DbInfo( String sub ) {
		readProperties( null, sub );
	}

	/**
	* コンストラクタ
	* 指定されたpropertiesで、接続先への情報を取得する
	* @param	prop		properties
	*/
	public DbInfo( ResourceBundle prop ) {
		readProperties( prop, null );
	}

	/**
	* コンストラクタ
	* 指定されたpropertiesからキーで検索して、接続先への情報を取得する
	* @param	prop		properties
	* @param	sub			キー
	*/
	public DbInfo( ResourceBundle prop, String sub ) {
		readProperties( prop, sub );
	}

	/**
	* コンストラクタ
	* 指定されたXMLから、接続先への情報を取得する
	* @param	config		DB接続情報
	*/
	public DbInfo( IConfiguration config )
		throws IllegalArgumentException, UnspecifiedPropertyException
	{
		String type = config.getAttributes().get( "type" );
		String var;
		if ( type.equals( "properties" )) {
			String key = null;
			key = config.get( "key", null );
			readProperties( null, key );
		} else if ( type.equals( "jndi" )) {
			context = config.get( "context", null );
			jndi = config.get( "url", null );
			source = config.get( "datasource" );
			user = config.get( "user", null );
			password = config.get( "password", null );
			var = config.get( "encrypted", null );
			if ( var != null && var.equals( "true" )) {
				encrypted = true;
			}
			encode = config.get( "encode", null );
			this.type = Type.DATASOURCE;
		} else if ( type.equals( "jdbc" )) {
			driver = config.get( "driver" );
			url = config.get( "url" );
			user = config.get( "user", null );
			password = config.get( "password", null );
			jar = config.get( "jar", null );
			var = config.get( "encrypted", null );
			if ( var != null && var.equals( "true" )) {
				encrypted = true;
			}
			encode = config.get( "encode", null );
			this.type = Type.JDBC;
		} else {
			throw( new IllegalArgumentException( "illegal type " + type ));
		}
		Iterator<IConfiguration> itr = config.getChilds( "property" );
		while ( itr.hasNext()) {
			IConfiguration c = (IConfiguration )itr.next();
			IConfiguration attr = c.getAttributes();
			String key = attr.get( "key" );
			String value = attr.get( "value" );
			props.put( key, value );
		}
	}

	/**
	* コンストラクタ
	* 指定された情報で、接続先への情報を取得する。
	* @param	ctx			JNDIコンテキスト
	* @param	jndi		JNDIのURL
	* @param	src			JNDIのデータソース名
	* @param	usr			ユーザ名
	* @param	pwd			パスワード
	* @param	enc			文字エンコード
	*/
	public DbInfo( String ctx, String jndi, String src,
					String usr, String pwd, String enc )
	{
		this( ctx, jndi, src, usr, pwd, false, enc );
	}

	/**
	* コンストラクタ
	* 指定された情報で、接続先への情報を取得する。
	* @param	ctx			JNDIコンテキスト
	* @param	jndi		JNDIのURL
	* @param	src			JNDIのデータソース名
	* @param	usr			ユーザ名
	* @param	pwd			パスワード
	* @param	encrypted	パスワード暗号化
	* @param	enc			文字エンコード
	*/
	public DbInfo( String ctx, String jndi, String src,
					String usr, String pwd, boolean encrypted, String enc )
	{
		this.context = ctx;
		this.jndi = jndi;
		this.source = src;
		this.user = usr;
		this.password = pwd;
		this.encrypted = encrypted;
		this.encode = enc;
		this.type = Type.DATASOURCE;
	}

	/**
	* コンストラクタ
	* 指定された情報で、接続先への情報を取得する。
	* @param	drv			JDBCドライバクラス名
	* @param	url			JDBCのURL
	* @param	usr			ユーザ名
	* @param	pwd			パスワード
	* @param	enc			文字エンコード
	*/
	public DbInfo( String drv, String url, String usr, String pwd, String enc )
	{
		this( drv, url, usr, pwd, false, enc, null );
	}
	/**
	* コンストラクタ
	* 指定された情報で、接続先への情報を取得する。
	* @param	drv			JDBCドライバクラス名
	* @param	url			JDBCのURL
	* @param	usr			ユーザ名
	* @param	pwd			パスワード
	* @param	encrypted	パスワード暗号化
	* @param	enc			文字エンコード
	*/
	public DbInfo( String drv, String url, String usr, String pwd,
					boolean encrypted, String enc )
	{
		this( drv, url, usr, pwd, encrypted, enc, null );
	}
	/**
	* コンストラクタ
	* 指定された情報で、接続先への情報を取得する。
	* @param	drv			JDBCドライバクラス名
	* @param	url			JDBCのURL
	* @param	usr			ユーザ名
	* @param	pwd			パスワード
	* @param	encrypted	パスワード暗号化
	* @param	enc			文字エンコード
	* @param	jar			jarファイル名
	*/
	public DbInfo( String drv, String url, String usr, String pwd,
					boolean encrypted, String enc, String jar )
	{
		this.driver = drv;
		this.url = url;
		this.user = usr;
		this.password = pwd;
		this.encrypted = encrypted;
		this.encode = enc;
		this.jar = jar;
		this.type = Type.JDBC;
	}

	private void readProperties( ResourceBundle prop, String sub ) {
		if ( prop == null ) {
			prop = getDefaultProperties();
		}
		infoProp = prop;
		getProperties( sub );
	}

	/**
	* デフォルトのpropertiesを取得する。
	* @return	デフォルトのproperties
	*/
	private ResourceBundle getDefaultProperties() {
		String name = System.getProperty( "DbEnv.properties" );
		if ( name == null ) {
			name = "properties.util.DbEnv";
		}
		ResourceBundle infoProp = PropertyResourceBundle.getBundle( name );
		return infoProp;
	}

	/**
	* propertiesから全ての情報を取得する。
	* @param	sub			キー
	*/
	private void getProperties( String sub ) {
		if ( infoProp == null ) {
			throw( new NullPointerException( "properties is null" ));
		}
		context = getProperty( sub, "DB_CONTEXT" );
		jndi = getProperty( sub, "DB_JNDI_URL" );
		source = getProperty( sub, "DB_SOURCE" );
		driver = getProperty( sub, "DB_DRIVER" );
		url = getProperty( sub, "DB_URL" );
		user = getProperty( sub, "DB_USER" );
		password = getProperty( sub, "DB_PASSWORD" );
		jar = getProperty( sub, "DB_JAR" );
		String var = getProperty( sub, "DB_ENCRYPTED" );
		if ( var != null && var.equals( "true" )) {
			encrypted = true;
		}
		encode = getProperty( sub, "DB_ENCODE" );

		if ( getDataSource() == null || getDataSource().length() == 0 ) {
			type = Type.JDBC;
		} else {
			type = Type.DATASOURCE;
		}
	}

	/**
	* propertiesから指定された情報を取得する。
	* @param	sub			キー
	* @param	key			キー
	*/
	private String getProperty( String sub, String key ) {
		String prm;
		if ( sub == null ) {
			prm = key;
		} else {
			prm = sub + "." + key;
		}
		String ret = null;
		try {
			ret = infoProp.getString( prm );
		} catch ( MissingResourceException me ) {
		}
		return ret;
	}

	/**
	* 接続タイプの取得
	* @return	接続タイプ
	*/
	public Type getType() {
		return type;
	}
	/**
	* Context Factoryの取得
	* @return	Context Factory
	*/
	public String getContext() {
		return context;
	}
	/**
	* JNDI URLの取得
	* @return	JNDI URL
	*/
	public String getJndiUrl() {
		return jndi;
	}
	/**
	* DataSourceの取得
	* @return	DataSource
	*/
	public String getDataSource() {
		return source;
	}
	/**
	* ドライバの取得
	* @return	ドライバ
	*/
	public String getDriver() {
		return driver;
	}
	/**
	* ＤＢ接続先の取得
	* @return	ＤＢ接続先
	*/
	public String getUrl() {
		return url;
	}
	/**
	* ユーザの取得
	* @return	ユーザ
	*/
	public String getUser() {
		return user;
	}
	/**
	* パスワードの取得
	* @return	パスワード
	*/
	public String getPassword() {
		return password;
	}
	/**
	* Jarの取得
	* @return	Jar
	*/
	public String getJar() {
		return jar;
	}
	/**
	* パスワードの暗号化の取得
	* @return	パスワードの暗号化
	*/
	public boolean isEncrypted() {
		return encrypted;
	}
	/**
	* パスワード暗号化キーの設定
	* @param	var		パスワード暗号化キー
	*/
	public void setEncryptKey( String var ) {
		this.encryptKey = var;
	}
	/**
	* パスワード暗号化キーの取得
	* @return	パスワード暗号化キー
	*/
	public String getEncryptKey() {
		return encryptKey;
	}
	/**
	* 文字エンコードの取得
	* @return	文字エンコード
	*/
	public String getEncode() {
		return encode;
	}
	/**
	* 接続プロパティの設定
	* @param	var		接続プロパティ
	*/
	void setProperties( Properties var ) {
		this.props = var;
	}
	/**
	* 接続プロパティの取得
	* @return	接続プロパティ
	*/
	public Properties getProperties() {
		return props;
	}

	/**
	* この出力先の文字列表現を返します。
	* @return	この出力先の文字列表現
	*/
	public String toString() {
		String ret = "";
		if ( getType() == Type.JDBC ) {
			// JDBC
			ret = "DRIVER[" + getDriver() + "] "
				+ "URL[" + getUrl() + "]";
			if ( getJar() != null ) {
				ret += " JAR[" + getJar() + "]";
			}
		} else {
			// JNDI
			ret = "CONTEXT[" + getContext() + "] "
				+ "JNDI_URL[" + getJndiUrl() + "] "
				+ "DATASOURCE[" + getDataSource() + "]";
		}
		if ( getUser() != null ) {
			ret += " USER[" + getUser() + "] "
				+ "PASSWORD[" + getPassword() + "]";
		}
		if ( getEncode() != null ) {
			ret += " ENCODE[" + getEncode() + "]";
		}
		ret += " PROP[" + props + "]";
		return ret;
	}

}
