package	jp.co.sds_corp.util.db;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.util.Map;
import java.util.TreeMap;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* Statementのラッパクラスです。
* SQL実行時に、SQL文をログファイルに出力します。
* @author	小見山　聡
*/
public class DbStatement implements Statement
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	Statement stmt = null;
	String	sql;
	Map<Integer,String>	map = new TreeMap<Integer,String>();
	/** SQL文出力フラグ */
	boolean	sqlOutput = true;

	public DbStatement( Statement stmt ) {
		this.stmt = stmt;
	}

	/**
	* SQL文出力フラグを設定します。
	* @param	var		SQL文出力フラグ
	*/
	public void setSqlOutput( boolean var ) {
		this.sqlOutput = var;
	}
	/**
	* SQL文出力フラグを取得します。
	* @return	SQL文出力フラグ
	*/
	public boolean isSqlOutput() {
		return this.sqlOutput;
	}

	protected void addParameter( int idx, String var ) {
		map.put( idx, var );
	}

	protected void clearParameter() {
		setSql( null );
		map.clear();
	}

	protected void printParameter() {
		if ( !isSqlOutput()) {
			return;
		}
		String msg = parameterString();
		logger.sql( msg);
	}

	protected String parameterString() {
		StringBuilder msg = new StringBuilder();
		msg.append( "sql[" + sql + "]" );

		if ( map.size() > 0 ) {
			msg.append( " {" );
			for ( String val : map.values()) {
				msg.append( val + ":" );
			}
		}
		msg.append( "}" );
		return msg.toString();
	}

	protected void setSql( String var ) {
		this.sql = var;
	}

	//---------------------------------------------------------------
	// Statement method.
	//---------------------------------------------------------------
	public ResultSet executeQuery( String sql ) throws SQLException {
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.executeQuery( sql );
	}

	public int executeUpdate( String sql ) throws SQLException {
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.executeUpdate( sql );
	}

	public void close() throws SQLException {
		clearParameter();
		stmt.close();
		stmt = null;
	}

	public int getMaxFieldSize() throws SQLException {
		return stmt.getMaxFieldSize();
	}

	public void setMaxFieldSize( int max ) throws SQLException {
		stmt.setMaxFieldSize( max );
	}

	public int getMaxRows() throws SQLException {
		return stmt.getMaxRows();
	}

	public void setMaxRows( int max ) throws SQLException {
		stmt.setMaxRows( max );
	}

	public void setEscapeProcessing( boolean enable ) throws SQLException {
		stmt.setEscapeProcessing( enable );
	}

	public int getQueryTimeout() throws SQLException {
		return stmt.getQueryTimeout();
	}

	public void setQueryTimeout( int seconds ) throws SQLException {
		stmt.setQueryTimeout( seconds );
	}

	public void cancel() throws SQLException {
		stmt.cancel();
	}

	public SQLWarning getWarnings() throws SQLException {
		return stmt.getWarnings();
	}

	public void clearWarnings() throws SQLException {
		stmt.clearWarnings();
	}

	public void setCursorName( String name ) throws SQLException {
		stmt.setCursorName( name );
	}

	public boolean execute( String sql ) throws SQLException {
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.execute( sql );
	}

	public ResultSet getResultSet() throws SQLException { 
		return stmt.getResultSet();
	}

	public int getUpdateCount() throws SQLException { 
		return stmt.getUpdateCount();
	}

	public boolean getMoreResults() throws SQLException { 
		return stmt.getMoreResults();
	}

	//--------------------------JDBC 2.0-----------------------------
	public void setFetchDirection( int direction ) throws SQLException {
		stmt.setFetchDirection( direction );
	}

	public int getFetchDirection() throws SQLException {
		return stmt.getFetchDirection();
	}

	public void setFetchSize( int rows ) throws SQLException {
		stmt.setFetchSize( rows );
	}

	public int getFetchSize() throws SQLException {
		return stmt.getFetchSize();
	}

	public int getResultSetConcurrency() throws SQLException {
		return stmt.getResultSetConcurrency();
	}

	public int getResultSetType()  throws SQLException {
		return stmt.getResultSetType();
	}

	public void addBatch( String sql ) throws SQLException {
		stmt.addBatch( sql );
	}

	public void clearBatch() throws SQLException {
		stmt.clearBatch();
	}

	public int[] executeBatch() throws SQLException {
		printParameter();
		clearParameter();
		return stmt.executeBatch();
	}

	public Connection getConnection()  throws SQLException {
		return stmt.getConnection();
	}

	//--------------------------JDBC 3.0-----------------------------
	public boolean getMoreResults( int current ) throws SQLException { 
		return stmt.getMoreResults( current );
	}

	public ResultSet getGeneratedKeys() throws SQLException { 
		return stmt.getGeneratedKeys();
	}

	public int executeUpdate( String sql, int[] columnIndexes )
		throws SQLException
	{ 
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.executeUpdate( sql, columnIndexes );
	}

	public int executeUpdate( String sql, String[] columnNames )
		throws SQLException
	{ 
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.executeUpdate( sql, columnNames );
	}

	public int executeUpdate( String sql, int autoGeneratedKeys )
		throws SQLException
	{ 
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.executeUpdate( sql, autoGeneratedKeys );
	}

	public boolean execute( String sql, int autoGeneratedKeys )
		throws SQLException
	{
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.execute( sql, autoGeneratedKeys );
	}

	public boolean execute( String sql, int[] columnIndexes )
		throws SQLException
	{
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.execute( sql, columnIndexes );
	}

	public boolean execute( String sql, String[] columnNames )
		throws SQLException
	{
		setSql( sql );
		printParameter();
		clearParameter();
		return stmt.execute( sql, columnNames );
	}

	public int getResultSetHoldability() throws SQLException { 
		return stmt.getResultSetHoldability();
	}

	//------------------------- JDBC 4.0 -----------------------------------
	public boolean isClosed() throws SQLException {
		return stmt.isClosed();
	}

	public void setPoolable( boolean var ) throws SQLException {
		stmt.setPoolable( var );
	}

	public boolean isPoolable() throws SQLException {
		return stmt.isPoolable();
	}

	//------------------------- Java7 -----------------------------------
	@Override
	public boolean isCloseOnCompletion() throws SQLException {
		return stmt.isCloseOnCompletion();
	}

	@Override
	public void closeOnCompletion() throws SQLException {
		stmt.closeOnCompletion();
	}

	//---------------------------------------------------------------
	// Wrapper method.
	//---------------------------------------------------------------
	public boolean isWrapperFor( Class<?> var ) throws SQLException {
		return stmt.isWrapperFor( var );
	}

	public <T> T unwrap( Class<T> var ) throws SQLException {
		return stmt.unwrap( var );
	}

}
