package	jp.co.sds_corp.util.log;

import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.LogRecord;

/**
* ログを出力します。
* @author	小見山　聡
*/
public class CmnLogger
{
	private static CmnLogFactory factory = CmnLogFactory.getInstance();
	private static CmnLogger instance = new CmnLogger();
	private static ConsoleLogger console = ConsoleLogger.getLogger(CmnLogger.class);
	Logger logger;

	private CmnLogger() {
	}

	CmnLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * @deprecated
	 */
	public static CmnLogger getInstance() {
		return instance;
	}

	public Logger getLogger() {
		return logger;
	}

	/**
	* log を出力します。
	* @param	logger	log を出力するLogger
	* @param	rec		出力するメッセージ
	*/
	public void log( LogRecord rec ) {
		if ( logger == null ) { // || isClosed() ) {
			console.log( rec.getLevel().toString() + ":" + rec.getMessage() );
			return;
		}
		if ( !logger.isLoggable( rec.getLevel() )) {
			return;
		}

		rec.setLoggerName( logger.getName() );
		rec.setThreadID( (int )Thread.currentThread().getId() );
/*
		rec.getSourceMethodName();
*/
		if ( factory.needCaller() ) {
			if ( rec instanceof CmnLogRecord ) {
				CmnLogRecord irec = (CmnLogRecord )rec;
				irec.setNeedCaller( true );
			}
		}

		String ebname = getEffectiveResourceBundleName( logger );
		if ( ebname != null ) {
			rec.setResourceBundleName( ebname );
			rec.setResourceBundle( findResourceBundle( ebname ));
		}
		logger.log( rec );
	}

	/**
	* log を出力します。
	* @param	logger	log を出力するLogger
	* @param	level	log の出力レベル
	* @param	msg		出力するメッセージ
	*/
	public void log( Level level, String msg ) {
		CmnLogRecord rec = new CmnLogRecord( level, msg );
		rec.setLoggerName( logger.getName() );

		String ebname = getEffectiveResourceBundleName( logger );
		if ( ebname != null ) {
			rec.setResourceBundleName( ebname );
			rec.setResourceBundle( findResourceBundle( ebname ));
		}
		log( rec );
	}

	private String getEffectiveResourceBundleName( Logger logger ) {
		Logger target = logger;
		while (target != null) {
			String rbn = target.getResourceBundleName();
			if (rbn != null) {
				return rbn;
			}
			target = target.getParent();
		}
		return null;
	}

	// 処理内容は簡略化
	private synchronized ResourceBundle findResourceBundle( String name ) {
		ResourceBundle ret = null;
		try {
			ret = ResourceBundle.getBundle( name );
		} catch ( Exception ex ) {
		}
		return ret;
	}

	/**
	* ログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	level	log の出力レベル
	* @return	対象のログが出力されるか否か
	*/
	public boolean isLoggable( Level level ) {
		if ( logger == null ) {
			return false;
		}
		return logger.isLoggable( level );
	}

	/**
	* traceログを出力するかどうかを返します。
	* @param	obj		log の出力を行なったClass。
	* @return	traceログを出力するかどうか
	*/
	public boolean isTraceEnable() {
		return isLoggable( Level.FINEST );
	}
	/**
	* debugログを出力するかどうかを返します。
	* @param	obj		log の出力を行なったClass。
	* @return	debugログを出力するかどうか
	*/
	public boolean isDebugEnable() {
		return isLoggable( Level.FINER );
	}
	/**
	* detailログを出力するかどうかを返します。
	* @param	obj		log の出力を行なったClass。
	* @return	detailログを出力するかどうか
	*/
	public boolean isDetailEnable() {
		return isLoggable( Level.FINE );
	}

	/**
	* trace log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void trace( String logMsg ) {
		log( Level.FINEST, logMsg );
	}
	/**
	* trace log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void trace( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.FINEST, logMsg + ":" + error );
		exception( Level.FINEST, t );
	}
	/**
	* debug log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void debug( String logMsg ) {
		log( Level.FINER, logMsg );
	}
	/**
	* debug log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void debug( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.FINER, logMsg + ":" + error );
		exception( Level.FINER, t );
	}
	/**
	* detail log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void detail( String logMsg ) {
		log( Level.FINE, logMsg );
	}
	/**
	* detail log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void detail( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.FINE, logMsg + ":" + error );
		exception( Level.FINE, t );
	}
	/**
	* info log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void info( String logMsg ) {
		log( Level.INFO, logMsg );
	}
	/**
	* info log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void info( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.INFO, logMsg + ":" + error );
		exception( Level.INFO, t );
	}
	/**
	* warn log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void warn( String logMsg ) {
		log( Level.WARNING, logMsg );
	}
	/**
	* warn log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void warn( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.WARNING, logMsg + ":" + error );
		exception( Level.WARNING, t );
	}
	/**
	* error log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void error( String logMsg ) {
		log( Level.SEVERE, logMsg );
	}
	/**
	* error log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void error( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.SEVERE, logMsg + ":" + error );
		exception( Level.SEVERE, t );
	}
	/**
	* fatal log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void fatal( String logMsg ) {
		log( Level.SEVERE, logMsg );
	}
	/**
	* fatal log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void fatal( String logMsg, Throwable t ) {
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( Level.SEVERE, logMsg + ":" + error );
		exception( Level.SEVERE, t );
	}

	/**
	* exception log を出力します。
	* @param	ex		出力する例外
	*/
	public void exception( Throwable ex ) {
		exception( Level.FINE, ex );
	}

	/**
	* exception log を出力します。
	* @param	obj		出力する例外
	*/
	public void exception( Level level, Throwable ex ) {
		CmnLogger cmnLogger = factory.getLogger( "jp.co.sds_corp.EXCEPTION" );
		if ( cmnLogger == null || !cmnLogger.isLoggable( level ) ||
			ex == null )
		{
			// ex.printStackTrace();
			return;
		}
		for ( ;; ) {
			StackTraceElement[] stacks = ex.getStackTrace();
			for ( StackTraceElement stack : stacks ) {
				cmnLogger.log( level, stack.toString() );
			}
			ex = ex.getCause();
			if ( ex == null ) {
				break;
			}
			cmnLogger.log( level, "Caused by: " + ex.toString() );
		}
	}

	/**
	* SQL log を出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	sql		出力するSQL
	*/
	public void sql( String sql ) {
		CmnLogger cmnLogger = factory.getLogger( "jp.co.sds_corp.SQL." + logger.getName() );
		cmnLogger.log( Level.FINE, sql );
	}


	/**
	 * @deprecated
	 */
	public boolean isLoggable( Object obj, Level level ) {
		CmnLogger logger = factory.getLogger(obj);
		return logger.isLoggable(level);
	}

	/**
	 * @deprecated
	 */
	public boolean isTraceEnable( Object obj ) {
		return isLoggable( obj, Level.FINEST );
	}
	/**
	 * @deprecated
	 */
	public boolean isDebugEnable( Object obj ) {
		return isLoggable( obj, Level.FINER );
	}
	/**
	 * @deprecated
	 */
	public boolean isDetailEnable( Object obj ) {
		return isLoggable( obj, Level.FINE );
	}
	/**
	 * @deprecated
	 */
	public void trace( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.trace(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void trace( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.trace(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void debug( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.debug(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void debug( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.debug(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void detail( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.detail(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void detail( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.detail(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void info( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.info(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void info( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.info(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void warn( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.warn(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void warn( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.warn(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void error( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.error(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void error( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.error(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void fatal( Object obj, String logMsg ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.fatal(logMsg);
	}
	/**
	 * @deprecated
	 */
	public void fatal( Object obj, String logMsg, Throwable t ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.fatal(logMsg, t);
	}
	/**
	 * @deprecated
	 */
	public void exception( Object obj, Throwable ex ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.exception(ex);
	}
	/**
	 * @deprecated
	 */
	public void exception( Object obj, Level level, Throwable ex ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.exception(level, ex);
	}
	/**
	 * @deprecated
	 */
	public void sql( Object obj, String sql ) {
		CmnLogger logger = factory.getLogger(obj);
		logger.sql(sql);
	}

	private CallerInfo getCallerInfo() {
		StackTraceElement stack[] = (new Throwable()).getStackTrace();
		int ix = 0;
		while ( ix < stack.length ) {
			StackTraceElement frame = stack[ix];
			String cname = frame.getClassName();
			if ( cname.equals( "jp.co.sds_corp.util.common.CmnLogger" )) {
				break;
			}
			ix++;
		}
		while ( ix < stack.length ) {
			StackTraceElement frame = stack[ix];
			String cname = frame.getClassName();
			if ( !cname.equals( "jp.co.sds_corp.util.common.CmnLogger" )) {
				CallerInfo ret = new CallerInfo( stack[ix] );
				return ret;
			}
			ix++;
		}
		return null;
	}

	static class CallerInfo {
		String	classname;
		String	methodname;

		CallerInfo( StackTraceElement stack ) {
			classname = stack.getClassName();
			methodname = stack.getMethodName()
					+ "(" + stack.getLineNumber() + ")";
		}

		String	getClassName() {
			return classname;
		}
		String	getMethodName() {
			return methodname;
		}
	}

}
