package	jp.co.sds_corp.util.log;

import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;

/**
* コンソールにログを出力します。
* @author	小見山　聡
* @version	$Id$
*/
public class ConsoleLogger
{
	private static ConsoleLogger instance = new ConsoleLogger();
	private PrintStream out;
	private Level level;
	String category;

	private ConsoleLogger() {
		out = System.out;
		level = Level.FINE;

		String var;
		var = System.getProperty( getClass().getName() + ".LEVEL" );
		if ( var != null ) {
			try {
				level = Level.parse( var );
			} catch ( Exception ex ) {
				out.println( "unknown level[" + var + "]." );
			}
		}
		// out.println( getClass().getName() + " level=" + level + "." );
	}

	ConsoleLogger(Object obj) {
		if (instance == null) {
			instance = new ConsoleLogger();
		}
		this.out = instance.out;
		this.level = instance.level;
		String category = getCategory(obj);
		this.category = category;
	}

	/**
	* このクラスの唯一のインスタンスを返します。
	* @return	インスタンス
	*/
	public static ConsoleLogger getLogger(Object obj) {
		ConsoleLogger logger = new ConsoleLogger(obj);
		return logger;
	}

	/**
	* @deprecated
	*/
	public static ConsoleLogger getInstance() {
		return instance;
	}

	public void setLevel( Level level ) {
		this.level = level;
	}

	public Level getLevel() {
		return this.level;
	}

	public void setOut(PrintStream out) {
		this.out = out;
	}

	private String getCategory( Object obj ) {
		String ret = null;
		if ( obj instanceof Class ) {
			ret = ((Class<?> )obj).getSimpleName();
		} else if ( obj instanceof String ) {
			ret = (String )obj;
		} else {
			ret = obj.getClass().getSimpleName();
		}

		return ret;
	}

	public boolean isLoggable( Level level ) {
		if ( level.intValue() < this.level.intValue() ) {
			return false;
		}
		return true;
	}

	public void print( String var ) {
		out.print( var );
	}

	public void println( String var ) {
		out.println( var );
	}

	public PrintStream printf( String format, Object... args ) {
		return out.printf( format, args );
	}

	/**
	* ログを出力します。<br>
	* @param	obj		log の出力を行なったClass。
	* @param	level	出力するレベル
	* @param	logMsg	出力するメッセージ
	*/
	public synchronized void log( Level level, String logMsg ) {
		if ( !isLoggable( level )) {
			return;
		}
		SimpleDateFormat sdf = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss.SSS" );
		long id = Thread.currentThread().getId();
		out.println( sdf.format( new Date() )
			+ " " + String.format( "%08X", id )
			+ " [" + level + " " + category + "] "
			+ logMsg );
	}

	/**
	* ログを出力します。<br>
	* @param	obj		log の出力を行なったClass。
	* @param	level	出力するレベル
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public synchronized void log( Level level, String logMsg,
			Throwable t )
	{
		if ( !isLoggable( level )) {
			return;
		}
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( level, logMsg + ":" + error );
		exception( t );
	}

	/**
	* ログを出力します。<br>
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public synchronized void log( String logMsg ) {
		log( Level.FINER, logMsg );
	}

	/**
	* ログを出力します。<br>
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public synchronized void log( String logMsg, Throwable t ) {
		log( Level.FINER, logMsg, t );
	}

	/**
	* debugログを出力します。<br>
	* FINERレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void debug( String logMsg ) {
		log( Level.FINER, logMsg );
	}
	/**
	* debugログを出力します。<br>
	* FINERレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void debug( String logMsg, Throwable t ) {
		log( Level.FINER, logMsg, t );
	}

	/**
	* infoログを出力します。<br>
	* INFOレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void info( String logMsg ) {
		log( Level.INFO, logMsg );
	}
	/**
	* infoログを出力します。<br>
	* INFOレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void info( String logMsg, Throwable t ) {
		log( Level.INFO, logMsg, t );
	}

	/**
	* errorログを出力します。<br>
	* SEVEREレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	public void error( String logMsg ) {
		log( Level.SEVERE, logMsg );
	}
	/**
	* errorログを出力します。<br>
	* SEVEREレベルのログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public void error( String logMsg, Throwable t ) {
		log( Level.SEVERE, logMsg, t );
	}

	/**
	* exceptionログを出力します。
	* @param	obj		出力する例外
	*/
	public synchronized void exception( Throwable obj ) {
		if ( obj == null ) {
			out.println( "no exception." );
			return;
		}
		for ( ;; ) {
			StackTraceElement[] stacks = obj.getStackTrace();
			for ( StackTraceElement stack : stacks ) {
				out.println( "\t" + stack.toString() );
			}
			obj = obj.getCause();
			if ( obj == null ) {
				break;
			}
			out.println( "Caused by: " + obj.toString() );
		}
	}

	/**
	* @deprecated
	*/
	public void debug( Object obj, String logMsg ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.FINER, logMsg );
	}
	/**
	* @deprecated
	*/
	public void debug( Object obj, String logMsg, Throwable t ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.FINER, logMsg, t );
	}

	/**
	* @deprecated
	*/
	public void info( Object obj, String logMsg ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.INFO, logMsg );
	}
	/**
	* @deprecated
	*/
	public void info( Object obj, String logMsg, Throwable t ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.INFO, logMsg, t );
	}

	/**
	* @deprecated
	*/
	public void error( Object obj, String logMsg ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.SEVERE, logMsg );
	}
	/**
	* @deprecated
	*/
	public void error( Object obj, String logMsg, Throwable t ) {
		ConsoleLogger logger = ConsoleLogger.getLogger(obj);
		logger.log( Level.SEVERE, logMsg, t );
	}

}
