package	jp.co.sds_corp.util.log;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.logging.ErrorManager;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;
import java.util.logging.StreamHandler;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
* ログ出力の基本クラスです。
* LogLoaderから初期化されることを前提としています。
*/
public class FileHandler extends StreamHandler implements IConfigurable
{
	private static final String DEFAULT_PATTERN = "%h/default.log";
	private static final int DEFAULT_LIMIT = 0;

	ConsoleLogger console = ConsoleLogger.getLogger(this);
	LogManager	manager;
	String	pattern;
	String	fileName;
	boolean	append = true;
	long	limit;
	File	outfile;
	OutputStream output;
	long	msgSize;
	int		count;

	public FileHandler() throws IOException {
		super();
		try {
			init();
		} catch ( IOException ex ) {
			console.error( "log handler initialize error.", ex );
			throw( ex );
		} catch ( RuntimeException ex ) {
			console.error( "log handler initialize error.", ex );
			throw( ex );
		}
	}

	protected void init() throws IOException {
		manager = LogManager.getLogManager();
		manager.checkAccess();
	}

	public void init( IConfiguration config ) throws Exception {
		List<IConfiguration> list = config.listChild( "param" );
		for ( IConfiguration ele : list ) {
			String name = ele.getAttributes().get( "name" );
			String value = ele.getAttributes().get( "value" );
			if ( name == null ) {
			} else
			if ( name.equals( "pattern" )) {
				if ( value == null || value.length() == 0 ) {
					value = DEFAULT_PATTERN;
				}
				this.pattern = value;
			} else
			if ( name.equals( "limit" )) {
				try {
					limit = Long.valueOf( value ).longValue();
				} catch ( NumberFormatException ex ) {
					limit = DEFAULT_LIMIT;
				}
			} else
			if ( name.equals( "append" )) {
				if ( value == null || value.length() == 0 ) {
					append = true;
				} else
				if ( value.equals( "false" )) {
					append = false;
				}
			} else {
				console.error( "unknown param name=" + name );
			}
		}

		try {
			initOutput();
		} catch ( IOException ex ) {
			getErrorManager().error( "output initialize error."
				+ " handler=" + getClass().getName(),
				ex, ErrorManager.OPEN_FAILURE );
		}
	}

	protected void initOutput() throws IOException {
		fileName = parseFileName( this.pattern );
		console.debug( "init log property "
					+ ":" + Thread.currentThread().getId() + ":"
					+ " pattern=" + this.pattern + " file=" + fileName
					+ " limit=" + this.limit + " append=" + this.append );
		outfile = new File( fileName );
		long size = outfile.length();
		OutputStream output = new MeasureOutputStream(
			new BufferedOutputStream(
				new FileOutputStream( outfile, this.append )), size );

		setOutputStream( output );
	}

    String parseFileName( String pattern ) {
		int cur = 0;
		int next = 0;
        String tempPath = System.getProperty( "java.io.tmpdir" );
        boolean tempPathHasSepEnd = ( tempPath == null ? false
				: tempPath.endsWith( File.separator ));

        String homePath = System.getProperty( "user.home" );
        boolean homePathHasSepEnd = ( homePath == null ? false
				: homePath.endsWith( File.separator ));

        StringBuilder sb = new StringBuilder();
        pattern = pattern.replace( '/', File.separatorChar );

        char[] value = pattern.toCharArray();
        while (( next = pattern.indexOf( '%', cur )) >= 0 ) {
            if ( ++next < pattern.length()) {
                switch ( value[next] ) {
                    case 't':
                        sb.append( value, cur, next - cur - 1 ).append( tempPath );
                        if ( !tempPathHasSepEnd ) {
                            sb.append( File.separator );
                        }
                        break;
                    case 'h':
                        sb.append( value, cur, next - cur - 1 ).append( homePath );
                        if ( !homePathHasSepEnd ) {
                            sb.append( File.separator );
                        }
                        break;
                    case 'T':
						long id = Thread.currentThread().getId();
                        sb.append( value, cur, next - cur - 1 )
							.append( String.format( "%08X", id ));
                        break;
                    case '%':
                        sb.append( value, cur, next - cur - 1 ).append( '%' );
                        break;
                    default:
                        sb.append( value, cur, next - cur + 1 );
                }
                cur = ++next;
            } else {
                // fail silently
            }
        }

        sb.append( value, cur, value.length - cur );

        return sb.toString();
    }

	@Override
	public synchronized void publish( LogRecord record ) {
		super.publish( record );
		flush();

		long size = 0;
		if ( output instanceof MeasureOutputStream ) {
			MeasureOutputStream mos = (MeasureOutputStream )output;
			size = mos.getLength();
		} else {
			count++;
			if ( record.getMessage() != null ) {
				msgSize += record.getMessage().length();
			}
			if ( msgSize < limit ) {
				if ( count < 10000 || ( count % 1000 ) != 0 ) {
					return;
				}
			}

			size = outfile.length();
		}

		if ( limit > 0 && size >= 0 && size > limit ) {
			AccessController.doPrivileged( new PrivilegedAction<Object>() {
				public Object run() {
					try {
						roll();
					} catch ( IOException ex ) {
						console.error( "log file roll error.", ex );
					}
					return null;
				}
			});
			msgSize = 0;
			count = 0;
		}
	}

	protected void roll() throws IOException {
		SimpleDateFormat sdf = new SimpleDateFormat( "yyMMddHHmmss" );
		File back = null;
		Calendar cal = Calendar.getInstance();
		Date date = new Date();
		for ( ;; ) {
			String backName = fileName + "." + sdf.format( date );
			back = new File( backName );
			if ( back.exists()) {
				cal.setTime( date );
				cal.add( Calendar.SECOND, 1 );
				date = cal.getTime();
			} else {
				break;
			}
		}
		super.close();

		File org = new File( fileName );
		boolean renamed = org.renameTo( back );
		if ( !renamed  ) {
			getErrorManager().error( "log file rename error.",
				null, ErrorManager.GENERIC_FAILURE );
		}
		try {
			console.debug( "reopen file"
					+ ":" + Thread.currentThread().getId() + ":"
					+ " file=" + fileName );
			outfile = new File( fileName );
			OutputStream output = new MeasureOutputStream(
				new BufferedOutputStream(
					new FileOutputStream( outfile, this.append )));
			setOutputStream( output );
		} catch ( FileNotFoundException ex ) {
			getErrorManager().error( "log file reopen error.",
				ex, ErrorManager.OPEN_FAILURE );
		}
	}

	protected void rollWithLock() throws IOException {
		String lockName = fileName + ".lck";
		File lockFile = new File( lockName );
		FileOutputStream lockStr = null;
		FileChannel channel = null;
		FileLock lock = null;
		try {
			lockStr = new FileOutputStream( lockFile );
			channel = lockStr.getChannel();
			if ( channel == null ) {
				getErrorManager().error( "lock file open error.",
						null, ErrorManager.GENERIC_FAILURE );
				return;
			}
			try {
				lock = channel.tryLock();
			} catch ( IOException ex ) {
				getErrorManager().error( "lock file lock error.",
						ex, ErrorManager.GENERIC_FAILURE );
			}
			if ( lock == null ) {
				try {
					lockStr.close();
					boolean deleted = lockFile.delete();
				} catch ( IOException ex ) {
				}
				return;
			}

			roll();

		} finally {
			try {
				if ( lock != null ) {
					lock.release();
				}
				if ( channel != null ) {
					channel.close();
				}
				if ( lockStr != null ) {
					lockStr.close();
					boolean deleted = lockFile.delete();
					if ( !deleted  ) {
						getErrorManager().error( "lock file delete error.",
							null, ErrorManager.GENERIC_FAILURE );
					}
				}
			} catch ( IOException ex ) {
				// ignore
				getErrorManager().error( "lock file release error.",
					ex, ErrorManager.GENERIC_FAILURE );
			}
			try {
				lockStr.close();
			} catch ( Exception ex ) {
			}
		}
	}

	@Override
	protected synchronized void setOutputStream( OutputStream os ) {
		this.output = os;
		super.setOutputStream( os );
	}

	protected synchronized OutputStream getOutputStream() {
		return this.output;
	}

}
