package	jp.co.sds_corp.util.math;


/**
* Jacobiの反復法による連立方程式の解法
*/
public class Jacobi
{
	private static double TINY  = 1.0E-20;
	private static double EPS   = 1.0E-6;
	private static int MAX_ITER = 100;

	public void setEPS(double _eps) {
		EPS = _eps;
	}
	public void setTINY(double _tiny) {
		TINY = _tiny;
	}
	public void setMAX_ITER(int _max_iter) {
		MAX_ITER = _max_iter;
	}

	public static void rotate(double [][] a, int i, int j, int k, int l, double c, double s) {
		double x = a[i][j];
		double y = a[k][l];
		a[i][j] = x * c - y * s;
		a[k][l] = x * s + y * c ;
	}

	/**
	* Jacobiの反復法による連立方程式の解法
	*/
	public static boolean solve(double [][] a, double [][] w) {
		int n = a.length;
		double s = 0, offdiag = 0;

		for (int j=0; j<n; j++) {
			for (int k=0; k<n; k++) { w[j][k] = 0; }
			w[j][j] = 1;
			s += a[j][j] * a[j][j];
			for (int k=j+1; k<n; k++) {
				offdiag += a[j][k] * a[j][k];
			}
		}
		double tolerance = EPS * EPS * (s / 2 + offdiag);
		int iter;
		for (iter = 1; iter<=MAX_ITER; iter++) {
			offdiag = 0;
			for (int j=0; j<n-1; j++) {
				for (int k=j+1; k<n; k++) {
					offdiag += a[j][k] * a[j][k];
				}
			}

			if (offdiag < tolerance) {
				break;
			}

			for (int j=0; j<n-1; j++) {
				for (int k=j+1; k<n; k++) {
					if (Math.abs(a[j][k]) < TINY) {
						continue;
					}
					double t = (a[k][k] - a[j][j]) / (2 * a[j][k]);
					if (t >= 0) {
						t = 1 / (t + Math.sqrt(t * t + 1));
					} else {
						t = 1 / (t - Math.sqrt(t * t + 1));
					}
					double c = 1 / Math.sqrt(t * t + 1);
					double u = t * c;
					t *= a[j][k];
					a[j][j] -= t;
					a[k][k] += t;
					a[j][k] = 0;
					int i;
					for (i=0; i<j; i++) {
						rotate(a, i, j, i, k, c, u);
					}
					for (i=j+1; i<k; i++) {
						rotate(a, j, i, i, k, c, u);
					}
					for (i=k+1; i<n; i++) {
						rotate(a, j, i, k, i, c, u);
					}
					for (i=0; i<n; i++) {
						rotate(w, j, i, k, i, c, u);
					}
				}
			}
		}

		if (iter > MAX_ITER) {
			return false;
		}
		for (int i=0; i<n-1; i++) {
			int k = i;
			double t = a[k][k];
			for (int j=i+1; j<n; j++) {
				if (a[j][j] > t) {
					k = j;
					t = a[k][k];
				}
			}
			a[k][k] = a[i][i];
			a[i][i] = t;

			double [] v = w[k];
			w[k] = w[i];
			w[i] = v;
		}
		return true;
	}
}
