package jp.co.sds_corp.util.syslog;

import java.io.Serializable;
import java.net.InetAddress;
import java.util.Locale;
import java.util.StringTokenizer;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.NoSuchCodeException;

/**
* syslogのmessage内容の解析ユーティリティです。
* @author	小見山　聡
*/
public class SyslogMsg implements Serializable
{
	private static final long serialVersionUID = 1L;
	private String		encode;
	private String		orgMsg;
	private String		msg;
	private String		orgtime;
	private String		tag;
	private String		ident;
	private Priority	priority;
	private Facility	facility;
	private CmnDate		date;
	private	InetAddress	address;

	/**
	* メッセージの内容でオブジェクトを初期化します。
	* @param	aByte	syslogのメッセージ内容
	* @param	nLen	メッセージ長
	*/
	public SyslogMsg( byte[] aByte, int nLen )
		throws NoSuchCodeException
	{
		this( aByte, nLen, "EUC-JP" );
	}

	/**
	* メッセージの内容でオブジェクトを初期化します。
	* @param	aByte	syslogのメッセージ内容
	* @param	nLen	メッセージ長
	*/
	public SyslogMsg( byte[] aByte, int nLen, String encode )
		throws NoSuchCodeException
	{
		this.encode = encode;
		int		nCode = 0;
		int		nStart = 0;
		if ( aByte[0] == '<' ) {
			for ( int nLoop = 0; nLoop < nLen; nLoop++ ) {
				if ( aByte[nLoop] == '>' ) {
					nStart = nLoop + 1;
					break;
				}
			}
		}
		if ( nStart > 0 ) {
			String	code = new String( aByte, 1, nStart - 2 );
			nCode = Integer.valueOf( code ).intValue();
			priority = Priority.parse( nCode & Priority.LOG_PRIMASK );
			facility = Facility.parse( nCode >>> 3 );
		}
		msg = orgtime = tag = ident = "";
		try {
			orgMsg = new String( aByte, nStart, nLen - nStart, encode );
			if (( nLen - nStart ) < 15 ) {
				msg = orgMsg;
				return;
			}
		} catch( Exception ex ) {
			ex.printStackTrace();
		}
		try {
			String	dt;
			// Solaris日時対応
			dt = orgMsg.substring( 0, 15 );
			try {
				CmnDate d = new CmnDate( dt, "MMM dd HH:mm:ss",
										new Locale( "en", "US" ) );
			} catch ( Exception ex ) {
				orgtime = "";
				msg = orgMsg;
				return;
			}
			orgtime = dt;

			String tmp2;
			dt = orgMsg.substring( 16, 40 );
			// Log4J日付対応
			try {
				CmnDate d = new CmnDate( dt, "yyyy/MM/dd HH:mm:ss" );
				tmp2 = orgMsg.substring( 41 );
			} catch ( Exception ex ) {
				tmp2 = orgMsg.substring( 16 );
			}
			StringTokenizer	st = new StringTokenizer( tmp2, ":" );
			if ( st.countTokens() <= 1 ) {
				// orgtime = "";
				msg = orgMsg;
				return;
			}
			tag = st.nextToken();
			st = new StringTokenizer( tmp2.substring( tag.length()+1 ), "]" );
			if ( st.countTokens() <= 1 ) {
				// orgtime = "";
				// tag = "";
				msg = orgMsg;
				return;
			}
			ident = st.nextToken() + "]";
/*
			ident = st.nextToken();
*/
			msg = tmp2.substring( tag.length()+2 + ident.length() );
		} catch( Exception ex ) {
			if ( orgMsg != null ) {
				msg = orgMsg;
			}
		}
	}

	/**
	* 指定された内容でオブジェクトを初期化します。
	* @param	nFacility	出力元の情報
	* @param	nPriority	重要度
	* @param	msg			メッセージ内容
	*/
	public SyslogMsg( int nFacility, int nPriority, String msg )
		throws NoSuchCodeException
	{
		this.facility = Facility.parse( nFacility );
		this.priority = Priority.parse( nPriority );
		this.msg = msg;
		this.orgMsg = msg;
	}

	/**
	* 指定された内容でオブジェクトを初期化します。
	* @param	facility	出力元の情報
	* @param	priority	重要度
	* @param	msg			メッセージ内容
	*/
	public SyslogMsg( String facility, String priority, String msg )
		throws NoSuchCodeException
	{
		this.facility = Facility.parse( facility );
		this.priority = Priority.parse( priority );
		this.msg = msg;
		this.orgMsg = msg;
	}

	/**
	* 指定された内容でオブジェクトを初期化します。
	* @param	facility	出力元の情報
	* @param	priority	重要度
	* @param	msg			メッセージ内容
	*/
	public SyslogMsg( Facility facility, Priority priority, String msg ) {
		this.facility = facility;
		this.priority = priority;
		this.msg = msg;
		this.orgMsg = msg;
	}

	/**
	* メッセージ内容に含まれている出力時間を取得します。
	* @return	出力時間
	*/
	public String getOrgTime() {
		return orgtime;
	}

	/**
	* メッセージ内容に含まれているTagを取得します。
	* Solarisのsyslogのみに対応しています。
	* @return	Tag
	*/
	public String getTag() {
		return tag;
	}

	/**
	* メッセージ内容に含まれているIdentを取得します。
	* Solarisのsyslogのみに対応しています。
	* @return	Ident
	*/
	public String getIdent() {
		return ident;
	}

	/**
	* 出力元の情報を取得します。
	* @return	出力元
	*/
	public Facility getFacility() {
		return facility;
	}

	/**
	* 重要度の情報を取得します。
	* @return	重要度
	*/
	public Priority getPriority() {
		return priority;
	}

	/**
	* メッセージ内容を取得します。
	* @return	メッセージ
	*/
	public String getMessage() {
		return msg;
	}

	/**
	* メッセージ内容を取得します。
	* @return	メッセージ
	*/
	public String getOriginalMessage() {
		return orgMsg;
	}

	/**
	* 出力元と重要度の情報をsyslogのフォーマットに変換します。
	* @return	出力元と重要度の情報
	*/
	public String getData() {
		String	str = null;
		int	code = ( facility.getValue() << 3 ) | priority.getValue();
		str = "<" + String.valueOf( code ) + ">";
		str = str + orgMsg;
		return str;
	}

	/**
	* 受信日時(date)を設定します。
	* @param	var		設定するデータ
	*/
	public void setDate( CmnDate var ) {
		this.date = var ;
	}
	/**
	* 受信日時(date)を取得します。
	* @return	dateの内容
	*/
	public CmnDate getDate() {
		return date;
	}

	/**
	* 送信元のIPアドレス(address)を設定します。
	* @param	var		設定するデータ
	*/
	public void setAddress( InetAddress var ) {
		this.address = var ;
	}
	/**
	* 送信元のIPアドレス(address)を取得します。
	* @return	addressの内容
	*/
	public InetAddress getAddress() {
		return address;
	}


}
